/*
 *
 * Name:        OpenDVE_procs.c
 *
 * Description: Provide simple OpenDVE functions to ISSamplePlugin*.
 *
 * Change Log:
 *
 ********************************************************************
 *                                                                  *
 * (C) Copyright InSoft, Inc. 1995 All rights reserved              *
 *                                                                  *
 * You have a royalty-free right to use, modify, and reproduce the  *
 * the Sample Files (and/or any modified version) in any way you    *
 * find useful, provided that you agree that InSoft has no warranty *
 * obligations or liability for any Sample OpenDVE Plugin Files     *
 * which are modified.                                              *
 *                                                                  *
 ********************************************************************
 */

#define XT

#include <stdio.h>
#include <sys/types.h>

#ifdef XVIEW
#include <xview/xview.h>
#endif

#ifdef XT
#include <X11/Intrinsic.h>
#endif

/*
 * OpenDVE related include files.
 */
#include <dve/dve.h> 
#include <dve/pluginManagement.h> 
#include <dve/clipboard.h>

/*
 * Include definition of samplePluginMessageStructure.
 */
#include "AudioPluginProtocol.h"

/*
 * Local callback funtions.
 */
static void ConferenceStarting(DveSession dveSession, DveEvent* event);
static void EngineShutDown(DveSession dveSession, DveEvent* event);
static void PluginControlMsg(DveSession dveSession, 
                             DveUserHandle sender,
                             DveDataPacket* packet);
static void MemberLeaving(DveSession dveSession, DveEvent* event);
static void NewMemberAdded(DveSession dveSession, DveEvent* event);
static void PluginMessageCallback(DveSession dveSession,
                                  DveUserHandle sender,
                                  DveDataPacket* packet);
DveClipboard InitializeOpenDVEClipboard(DveSession dveSession, 
                                        char *clipboardName,
                                        DveUserHandle userHandle);

/*
 * Needed variables.
 */
DveSession dveSession = NULL;
DveClipboard clipboard = NULL;

/*
 * This is here for the initialization of OpenDVE.
 */

#ifdef XVIEW
   DveSession 
   InitializeOpenDVE(Xv_opaque win, DvePort pluginPort)
#endif

#ifdef XT
   DveSession 
   InitializeOpenDVE(XtAppContext app_context, DvePort pluginPort)
#endif
{
   /*
    * Open the DVE Session.
    */
   if (!(dveSession=dveOpenSession(pluginPort)))
   {
      fprintf(stderr, "Initialization of OpenDVE Failed...exiting.\n\7");
      switch (dveError)
	{
	case dveErrOutOfMemory:
	  fprintf(stderr, "Error Condition: [Low memory.]\n");
	  break;
	  
	case dveErrNotFound:
	  fprintf(stderr, 
		  "Error Condition: [Conference Engine not found.]\n");
	  break;
	  
	case dveErrConnRefused:
	  fprintf(stderr, 
		  "Error Condition: [Engine refused connection.]\n");
	  break;
	default:
	  fprintf(stderr, "Unknown Error Condition.\n");
	  break;
	}

      return(NULL);
   }

   /*
    * Set the callbacks.  
    */
   dveSetEventCallback(dveSession,dveEventTagStarting,ConferenceStarting);
   dveSetEventCallback(dveSession, dveEventTagEngineShutdown, EngineShutDown);
   dveSetEventCallback(dveSession,dveEventTagMemberLeft,MemberLeaving);
   dveSetEventCallback(dveSession,dveEventTagNewMember,NewMemberAdded);

   dveSetUserCallback(dveSession, pluginPort, PluginMessageCallback);
   dveSetUserCallback(dveSession,dvePluginManagementProtocol, PluginControlMsg);

   /*
    * Receive the first event.
    */
   dveReceiveEvent(dveSession, dveEventTagStarting);

#ifdef XVIEW
   /*
    * Register with XView.
    */
   if (dveRegisterWithXView(dveSession, win)<0)
   {
      dveCloseSession(dveSession);
      return(NULL);
   }
#endif

#ifdef XT
   /*
    * Register with Xt.
    */
   if (dveRegisterWithXt(dveSession, app_context)<0)
   {
      dveCloseSession(dveSession);
      return(NULL);
   }
#endif

   dveReportPresence(dveSession);

   return(dveSession);
}

/*
 * Orderly shutdown of the OpenDVE session.
 */
static void
QuitPlugin(DveSession dveSession)
{
   CleanupPlugin();
   dveCloseSession(dveSession);

#ifdef XT
   dveUnregisterWithXt(dveSession);
#endif

#ifdef XVIEW
   dveUnregisterWithXView(dveSession);
#endif
   exit(0);
}

/*
 * QuitPlugin if requested.
 */
static void 
PluginControlMsg(DveSession dveSession, DveUserHandle sender,
                 DveDataPacket* packet)
{
   switch ((DvePluginManagementMsgTag)packet->tag)
   {
      case dvePluginManagementMsgTagQuit:
         QuitPlugin(dveSession);
         break;
      case dvePluginManagementMsgTagHide:
         HidePlugin();
         break;
      case dvePluginManagementMsgTagShow:
         ShowPlugin();
         break;
   }
}

/*
 * If we notice that the conference engine has shutdown, we should
 * also shutdown politely.
 */
static void
EngineShutDown(DveSession dveSession, DveEvent* event)
{
   QuitPlugin(dveSession);
}

/*
 * This is called for the kickoff of the conference. This is needed
 * for tools that need to have their state updated for late starters.
 * If the plugin doesn't require this, then it could just be 
 * commented out. 
 */
static void 
ConferenceStarting(DveSession dveSession, DveEvent* event)
{
   DveEventStartingArg_Member* members=event->arg.starting.members;
   int numMembers=event->arg.starting.numMembers;
   DveConfParticipant* senior=NULL;
   int self=event->arg.starting.self;
   int i;
  
   /*
    * Cycle through conferenced members to determine the 
    * senior member.
    */
   for (i=0; i<numMembers; i++)
   {
      if (i==self)
         continue;
        
      if ((!senior) || (members[i].part.userHandle<senior->userHandle))
         senior=&(members[i].part);
   }   
		
   if (!senior)
   {
      return;
   } 

   /*
    * Now query the senior member for his current plugin status.
    */
   QueryPluginStatus(senior);
}   

/*
 * This is for when a conference member leaves the session.
 */ 
static void 
MemberLeaving(DveSession dveSession, DveEvent* event)
{
   MemberLeft(event->arg.memberLeft.member);
}   

/*
 * This is for when a conference member is added to the session.
 */ 
static void 
NewMemberAdded(DveSession dveSession, DveEvent* event)
{
   MemberAdded(event->arg.memberLeft.member);
}   

/*
 *  This is for an orderly shutdown of the session.
 */
void
ClosePluginSession(DveSession dveSession)
{
   CleanupPlugin();
#ifdef XT
   dveUnregisterWithXt(dveSession);
#endif

#ifdef XVIEW
   dveUnregisterWithXView(dveSession);
#endif
   dveCloseSession(dveSession);
}

/*
 * Callback for all user events on port number, SAMPLE_PLUGIN_PORT.
 */
static void 
PluginMessageCallback(DveSession dveSession, DveUserHandle sender, DveDataPacket* packet)
{
  int data_size;

  data_size = packet->size - sizeof(DveDataPacket) + 1; 
  ProcessPluginMessage(packet->tag, packet->data, data_size, sender);
}

/*
 * Passses data from this plugin to other conferenced members.
 */
int
SendPluginMessageGeneric(DveSession dveSession, u_int32 protocol, DvePort port,
			u_int32 messageType, int dest, void *data, u_int32 size)
{
   DveDataPacket *packet;
   DveConfParticipant member;
   int returnCode;

   packet = (DveDataPacket*) dveMallocDataPacket(dveSession, size);
   packet->tag = (u_int32)messageType;
   packet->pluginProtocol = protocol;
   packet->destPort = port;

   DvePartListClrAll(&(packet->destParts));

   if (dest < 0)
   {
      DvePartListSetAll(&(packet->destParts));
   }
   else
   {
      DvePartListSet(dest,&(packet->destParts));
   }

   if (size > 0)
      memcpy(packet->data, data, size);

   returnCode = dveSendDataPacket(dveSession, packet);

   dveFreeDataPacket(packet); 
   return(returnCode);
}

/* 
 * Send a plugin message to all users currently conferenced.
 */
int
SendPluginMessage(DveSession dveSession, DvePort port, u_int32 messageType,
				void *data, u_int32 size)
{
   return(SendPluginMessageGeneric(dveSession, port, port, 
                                   messageType, -1, data, size));
}

/*
 * Send a plugin message to the identified user.
 */
int
SendPluginMessageToMember(DveSession dveSession, DvePort port, 
		DveUserHandle member, u_int32 messageType, void *data, 
		u_int32 size)
{
   return(SendPluginMessageGeneric(dveSession, port, port,
                                   messageType, member, data, size));
}

/*
 * Send a message to myself.
 */
int
SendLocalPluginMessage(DveSession dveSession, DvePort port, 
			u_int32 messageType, void *data, u_int32 size)
{
   return(SendPluginMessageGeneric(dveSession, port, port, 
                                   messageType, 0, data, size));
}

/*
 * Send a message to conference management.
 */
int
SendPluginManagementMessage(DveSession dveSession, DvePort port, 
						u_int32 messageType)
{
   return(SendPluginMessageGeneric(dveSession, dvePluginManagementProtocol, port,
                                   messageType, 0, NULL, 0));
}

